<?php
// Autor (c) Miroslav Novak, www.platiti.cz
// Pouzivani bez souhlasu autora neni povoleno
// #Ver:PRV092-0-ga567b492:2022-01-28#

require_once("UniModul.php");
require_once("UniAdapterModel.php");

class UniAdapterController extends Mage_Core_Controller_Front_Action
{
	protected $uniModulName = null; //predefinuje binder
	protected $uniModul = null;
	protected $subMethod = '';

	protected function initUniModul() {
		if ($this->uniModul == null) {
			if (is_null($this->uniModulName)) return;
			$uniFact = new UniModulFactory();
			$configSetting = $this->getConfigDataUni($this->uniModulName, $this->subMethod);
			$configSetting->uniModulConfig->funcProcessReplyStatus = array($this, 'funcProcessReplyStatus');
			$this->uniModul = $uniFact->createUniModul($this->uniModulName, $configSetting, $this->subMethod);
		}
	}

	protected function getConfigDataUni($uniModulName, $subMethod) {
		$model = new UniAdapterModel($uniModulName, $subMethod);
		return $model->getConfigDataUniExternal($uniModulName, $subMethod);
	}

    public function redirectAction() {
    	BeginUniErr();
		$orderIncrementId = $this->getRequest()->getParam('orderId');
		$subMethod = '';
		if ($orderIncrementId === null) {
			$checkout = Mage::getSingleton('checkout/session');
			$orderIncrementId = $checkout->getLastRealOrderId();
			$subMethod = $checkout->getUniAdapterSubmethod();
		}

		$order = Mage::getModel('sales/order')->loadByIncrementId($orderIncrementId);
		$orderToPayInfo = $this->getOrderToPayInfo($order, $subMethod);
		$this->initUniModul();

		$redirectAction = $this->uniModul->gatewayOrderRedirectAction($orderToPayInfo);

		if (!$order->getEmailSent()) {
			$order->sendNewOrderEmail();
			$order->setIsCustomerNotified(true);
			$order->save();
		}
		if (!empty($GLOBALS['UniMagento_NoStatusChangeOnFailedPayment'])) {
			$code = strtolower($this->uniModulName) . 'binder';
			$retryUrl = Mage::getUrl($code . '/' . $code . '/retry/orderId/' . $orderToPayInfo->shopPairingInfo);
			if (!empty($redirectAction->orderReplyStatus->successHtml)) {
				$comment = $redirectAction->orderReplyStatus->successHtml;
			} else {
				$comment = $this->__("If you were unable to make the payment, click on: <a href='%s'>RETRY PAYMENT</a>", $retryUrl);
			}
			$order->sendOrderUpdateEmail(true, $comment);
			$order->save();
		}

		if ($redirectAction->orderReplyStatus != null) {   // okamzita odpoved muze byt i zaroven s redirektem - napr. Cetelem
			$frontend_redir = $redirectAction->redirectUrl==null && $redirectAction->redirectForm==null;
			$this->processReplyStatus($redirectAction->orderReplyStatus, $frontend_redir);
		}
		if ($redirectAction->redirectUrl != null) {
			Mage::app()->getResponse()->setRedirect($redirectAction->redirectUrl);
		} else if ($redirectAction->redirectForm != null) {
			$this->uniModul->formRedirect($redirectAction->redirectForm);
		}
    	EndUniErr();
    }

    public function replyAction() {
    	BeginUniErr();
    	$this->initUniModul();
		$orderReplyStatus = $this->uniModul->gatewayReceiveReply($this->getLanguage());
		$this->processReplyStatus($orderReplyStatus, true);
		EndUniErr();
    }

	 public function retryAction() {
    	BeginUniErr();
		$this->redirectAction();
    	EndUniErr();
    }

    public function notifyAction() {
    	BeginUniErr();
    	$this->initUniModul();
    	$orderReplyStatus = $this->uniModul->gatewayReceiveNotification();
    	$this->processReplyStatus($orderReplyStatus, false);
    	EndUniErr();
    }

	 public function callbackAction() {
    	BeginUniErr();
		$this->initUniModul();
		parse_str($_SERVER['QUERY_STRING'], $cbArgs);
		$this->uniModul->processCallbackRequest($_GET['_callbackName'], $cbArgs);
    	EndUniErr();
    }

	protected function getOrderToPayInfo($order, $subMethod) {
		$orderToPayInfo = new OrderToPayInfo();

		$orderToPayInfo->subMethod = $subMethod;

		$order_id = $order->getRealOrderId();
        $billing  = $order->getBillingAddress();
		if ($billing == null) {
			$billing = $order->getShippingAddress();
		}

		$orderToPayInfo->shopPairingInfo = $order_id;
		$orderToPayInfo->shopOrderNumber = $order_id;

		$orderToPayInfo->amount = $order->getGrandTotal();
		$orderToPayInfo->currency = $order->getOrderCurrencyCode();
		$orderToPayInfo->uniAdapterData = array('order_total'=>$order->getBaseGrandTotal());

		$description = Mage::app()->getStore()->getFrontendName();
		$products = array();
		$cartItems = [];
		foreach ($order->getAllVisibleItems() as $item) {
			$products[] = $item->getName();

			$cartItem = new UniCartItem();
			$cartItem->type = UniCartItemType::commodity;
			$cartItem->name = $item->getName();
			$cartItem->quantity = $item->getQtyToShip();
			$cartItem->unitPrice = $item->getPriceInclTax();
			$cartItem->taxRate = round($item->getTaxPercent());
			$cartItems[] = $cartItem;
		}
		$orderToPayInfo->description = $description;

		$cartItem = new UniCartItem();
		$cartItem->type = UniCartItemType::delivery;
		$cartItem->name = 'shipping / doprava';
		$cartItem->quantity = 1;
		$cartItem->unitPrice = $order->getShippingInclTax();
		$cartItem->taxRate = $order->getShippingAmount() == 0 ? 0 : round(100 * $order->getShippingTaxAmount() / $order->getShippingAmount());
		$cartItems[] = $cartItem;

		$discountAmount = $order->getDiscountAmount();
		if ($discountAmount != 0) {
			// secteni pres sazby
			$rozdeleni = array_reduce($cartItems, function($a, $ci) {
				$a[(string)$ci->taxRate] = $ci->unitPrice * $ci->quantity + (isset($a[$ci->taxRate]) ? $a[$ci->taxRate] :0);
				return $a;
			}, array());


			// rovnomerne rozpocteni slevy na pasma
			$totalBeforeDiscount = $orderToPayInfo->amount - $discountAmount;
			foreach($rozdeleni as $tax=>$amt) {
				$cartItem = new UniCartItem();
				$cartItem->type = UniCartItemType::discount;
				$cartItem->name = 'discount / sleva';
				$cartItem->quantity = 1;
				$cartItem->unitPrice = $discountAmount * $amt / $totalBeforeDiscount;
				$cartItem->taxRate = $tax;
				$cartItems[] = $cartItem;
			}


		}


		$orderToPayInfo->cartItems = $cartItems;

		$customerData = new CustomerData();
		try {
		$customerData->first_name = ($billing->getFirstname() ? $billing->getFirstname() : $order->getCustomerFirstname());
		$customerData->last_name = ($billing->getLastname() ? $billing->getLastname() : $order->getCustomerLastname());
		$customerData->email = ($billing->getEmail() ? $billing->getEmail() : $order->getCustomerEmail());
		} catch (Exception $ex) {
			$customerData->first_name = ($order->getCustomerFirstname());
			$customerData->last_name = ($order->getCustomerLastname());
			$customerData->email = ($order->getCustomerEmail());

		}
		$customerData->street = $billing->getStreet(-1);
		$customerData->city = $billing->getCity();
		$customerData->post_code = $billing->getPostcode();
		$customerData->country = $billing->getCountryModel()->getIso2Code();
		$customerData->phone = $billing->getTelephone();
		$customerData->identifier = $billing->getCustomer_id();  // pro guesta je to null
		$orderToPayInfo->customerData = $customerData;

		$code = strtolower($this->uniModulName) . 'binder';
		$orderToPayInfo->notifyUrl = Mage::getUrl($code . '/' . $code . '/notify');
		$orderToPayInfo->replyUrl = Mage::getUrl($code . '/' . $code . '/reply');
		$orderToPayInfo->uniModulDirUrl = Mage::getBaseUrl(Mage_Core_Model_Store::URL_TYPE_WEB) . "UniModul/";

		$orderToPayInfo->language = $this->getLanguage();

		$model = new UniAdapterModel();
		$orderToPayInfo->currencyRates = $model->getCurrencyRates();

		//$orderToPayInfo->uniAdapterData = $session->getId ();  //mozna bych tam mohl strcit celou session i s daty

		return $orderToPayInfo;
	}

	public function funcProcessReplyStatus($orderReplyStatus) {
				$this->processReplyStatus($orderReplyStatus, false);
	}

	function processReplyStatus($orderReplyStatus, $frontend_redir) {


		$cfg = $this->getConfigDataUni($this->uniModul->name, $this->uniModul->subMethod)->configData;

		BeginSynchronized();

		$order = Mage::getSingleton('sales/order');
		$order->loadByIncrementId($orderReplyStatus->shopPairingInfo);
		if (!$order->getId()) {
			// TODO  nacteni se nepovedlo...
			user_error("nacteni se nepovedlo");
			return;
		}


		if ($orderReplyStatus->orderStatus == OrderStatus::$failedRetriable || $orderReplyStatus->orderStatus == OrderStatus::$failedFinal) {
			if (!$order->getEmailSent()) {
				$order->sendNewOrderEmail();
				$order->setIsCustomerNotified(true);
				$order->save();
			}
			if (!empty($GLOBALS['UniMagento_NoStatusChangeOnFailedPayment'])) {
				$this->uniModul->logger->writeLog("ADAPTER: failed* => no effect when global UniMagento_NoStatusChangeOnFailedPayment set, shop order number ".$order->getRealOrderId() . ', sending email for retry payment');
				if (empty($orderReplyStatus->uniAdapterData['retryEmailSent'])) {
					$code = strtolower($this->uniModulName) . 'binder';
					$retryUrl = Mage::getUrl($code . '/' . $code . '/retry/orderId/' . $orderReplyStatus->shopPairingInfo);
					$comment = $this->__("Order payment failed, please try again: <a href='%s'>RETRY PAYMENT</a>", $retryUrl);
					$order->sendOrderUpdateEmail(true, $comment);
					$order->save();
					$transactionRecord = $this->uniModul->getOrderTransactionRecordFromDbUnique($orderReplyStatus->gwOrderNumber);
					$this->uniModul->updateUniAdapterDataInDb($transactionRecord->transactionPK, array('retryEmailSent' => true) + $transactionRecord->uniAdapterData);
					$this->uniModul->logger->writeLog("ADAPTER: DEBUG: retry payment email sent");
				} else {
					//$this->uniModul->logger->writeLog("ADAPTER: DEBUG: retry payment email not sent");
				}
			} else if (!in_array($order->getState(), array(Mage_Sales_Model_Order::STATE_NEW, Mage_Sales_Model_Order::STATE_CANCELED, Mage_Sales_Model_Order::STATE_PENDING_PAYMENT))) {
				$this->uniModul->logger->writeLog("ADAPTER: failed* => no effect in orderState " . $order->getState() . ", shop order number ".$order->getRealOrderId());
			} else {
				$this->uniModul->logger->writeLog("ADAPTER: failed* => Canceling order ".$order->getRealOrderId());

				$comment = $this->uniModul->dictionary->get('orderDetailGwOrdNum', $this->getLanguage()).': '.$orderReplyStatus->gwOrderNumber;;
				if ($orderReplyStatus->forexNote !== null) {
					$comment = "<br/>".$this->uniModul->dictionary->get('forexNoteLabel', $this->getLanguage()). ': ' . $orderReplyStatus->forexNote;
				}
				$order->registerCancellation($comment);
				$order->setStatus($cfg['orderStatusFailed']);
				$order->sendOrderUpdateEmail(true);
				$order->save();
			}


		} else if ($orderReplyStatus->orderStatus == OrderStatus::$successful ) {

			if (!in_array($order->getState(), array(Mage_Sales_Model_Order::STATE_NEW, Mage_Sales_Model_Order::STATE_CANCELED, Mage_Sales_Model_Order::STATE_PENDING_PAYMENT))) {

				$this->uniModul->logger->writeLog("ADAPTER: successful => no effect in orderState " . $order->getState() . ", shop order number ".$order->getRealOrderId());

			} else {

				$this->uniModul->logger->writeLog("ADAPTER: successful => registerCaptureNotification for shop order number ".$order->getRealOrderId());

				$comment = "";
				if ($orderReplyStatus->forexNote !== null) {
					$comment = $this->uniModul->dictionary->get('forexNoteLabel', $this->getLanguage()). ': ' . $orderReplyStatus->forexNote;
				}

				$payment = $order->getPayment();
				$payment->setTransactionId($orderReplyStatus->gwOrderNumber)
					->setIsTransactionClosed(0)
					->registerCaptureNotification($orderReplyStatus->uniAdapterData['order_total'], !empty($GLOBALS['UniMagento_skipFraudDetection']));
					if ($comment != "") {
						$payment->setPreparedMessage($comment);
					}
				$order->setStatus($cfg['orderStatusSuccessfull']);
				$order->save();

				$invoice = $payment->getCreatedInvoice();
				if ($invoice && !$order->getEmailSent()) {
					$order->sendNewOrderEmail();
					$order->setIsCustomerNotified(true);
				}
				$order->sendOrderUpdateEmail(true);
				$order->save();
			}


		} else if ($orderReplyStatus->orderStatus == OrderStatus::$pending ) {

			if (!in_array($order->getState(), array(Mage_Sales_Model_Order::STATE_NEW))) {

				$this->uniModul->logger->writeLog("ADAPTER: pending => no effect in orderState " . $order->getState() . ", shop order number ".$order->getRealOrderId());

			} else {

				$this->uniModul->logger->writeLog("ADAPTER: pending => setState STATE_PENDING_PAYMENT for shop order number ".$order->getRealOrderId());

				$comment = $this->uniModul->dictionary->get('orderDetailGwOrdNum', $this->getLanguage()).': '.$orderReplyStatus->gwOrderNumber;;
				if ($orderReplyStatus->forexNote !== null) {
					$comment = "<br/>".$this->uniModul->dictionary->get('forexNoteLabel', $this->getLanguage()). ': ' . $orderReplyStatus->forexNote;
				}

				$payment =  $order->getPayment()
					->setTransactionId($orderReplyStatus->gwOrderNumber)
					->setIsTransactionClosed(0);
				$order->setState(Mage_Sales_Model_Order::STATE_PENDING_PAYMENT, $cfg['orderStatusPending'], $comment, true);
				if (!$order->getEmailSent()) {
					$order->sendNewOrderEmail();
					$order->setIsCustomerNotified(true);
				}
				$order->save();
			}

		} else if ($orderReplyStatus->orderStatus == OrderStatus::$invalidReply) {
			$this->uniModul->logger->writeLog("ADAPTER: invalidReply => no effect");
		} else {
			$this->uniModul->logger->writeLog("ADAPTER: unknown orderStatus: " . $orderReplyStatus->orderStatus);
			user_error("ADAPTER: unknown orderStatus: " . $orderReplyStatus->orderStatus);
		}

		EndSynchronized();

		if ($frontend_redir) {
			if ($orderReplyStatus->orderStatus == OrderStatus::$pending || $orderReplyStatus->orderStatus == OrderStatus::$successful) {

				// zrusime kosik, pokdu byl predtim obnoven viz nize
				$session = Mage::getSingleton('checkout/session');
				$lastQuoteId = $session->getLastQuoteId();
				$quote = Mage::getModel('sales/quote')->load($lastQuoteId);
				$quote->setIsActive(false)->save();

				Mage::getSingleton('core/session')->setData('UniModulSuccessHtmlUniModul', $this->uniModulName);
				Mage::getSingleton('core/session')->setData('UniModulSuccessHtml', $orderReplyStatus->successHtml);
				$this->_redirect('checkout/onepage/success', array('_secure'=>true));
			} else {

				// obnovime kosik
				$session = Mage::getSingleton('checkout/session');
				$lastQuoteId = $session->getLastQuoteId();
				$quote = Mage::getModel('sales/quote')->load($lastQuoteId);
				$quote->setIsActive(true)->save();

				$this->loadLayout();
				$this->getLayout()->getBlock('root')->setTemplate('page/1column.phtml');
				$block = $this->getLayout()->createBlock(
					'Mage_Checkout_Block_Onepage_Failure',
					'newpage',
					array('template' => 'uniadapter_layout/failure.phtml')
				);

				$code = strtolower($this->uniModulName) . 'binder';
				$retryUrl = Mage::getUrl($code . '/' . $code . '/retry/orderId/' . $orderReplyStatus->shopPairingInfo);
				$block->assign(array('retryUrl' => $retryUrl, 'errorMessage' => $orderReplyStatus->resultText, "__" => array($this, "__")));
				$this->getLayout()->getBlock('content')->append($block);
				BeginUniErr(E_UNIERR_DEFAULT & ~E_WARNING); // v renderLayoutu zachytavme jinak chyby ve @vsprintf()
				$this->renderLayout();
				EndUniErr();
			}
		}
	}


	protected function getLanguage() {
		return substr(Mage::app()->getLocale()->getLocaleCode(), 0, 2);
	}

}
